import textpack from './textpack';

it('Randomizes text from a seed', () => {
    expect(textpack({
        value: 'Hello {target}',
        scope: {
            target: ['a', 'b', 'c'],
        },
    }, 'seed2')).toEqual('Hello a');
});

it('Prioritizes the nearest scope definition', () => {
    expect(textpack({
        value: '{sub}',
        scope: {
            a: 'world',
            target: '{a}',
            sub: {
                value: 'Hello {target}',
                scope: {
                    a: 'local',
                },
            },
        },
    })).toEqual('Hello world');
});

it('Invokes a src', () => {
    expect(textpack({
        value: 'Hello {capitalize target}',
        scope: {
            target: 'world',
            'capitalize': ([s]) => s.slice(0, 1).toUpperCase() + s.slice(1),
        },
    })).toEqual('Hello World');
});

it('Receives src arguments', () => {
    expect(textpack({
        value: 'Hello {decorate adjective target}',
        scope: {
            adjective: 'wonderful',
            target: 'world',
            decorate: 'to the {0} {1}',
        },
    })).toEqual('Hello to the wonderful world');
});

it('Defines a src with named parameters', () => {
    expect(textpack({
        value: 'Hello {decorate adjective noun}',
        scope: {
            adjective: 'wonderful',
            noun: 'world',
            decorate: {
                value: '{x} {y}',
                params: ['x', 'y'],
            },
        },
    })).toEqual('Hello wonderful world');
});

it('Only evaluates src arguments once', () => {
    expect(textpack({
        value: '{repeat message}',
        scope: {
            message: ['Hello', 'Hi', 'World', 'Place'],
            repeat: '{0} {0}',
        },
    }, 'seed3')).toEqual('Hi Hi');
});

// it('Invokes a src argument', () => {
//     expect(textpack({
//         value: 'Hello {decorate target}',
//         scope: {
//             target: '{0}',
//             selector: 'world',
//             decorate: 'to the {0 selector}',
//         },
//     })).toEqual('Hello to the world');
// });

it('Selects a key from a list', () => {
    expect(textpack({
        value: 'Hello {decorate target}',
        scope: {
            target: ['{{selectorA}}', '{{selectorB}}'],
            selectorA: 'world',
            selectorB: ['place', 'adventure'],
            decorate: 'to the {0}',
        },
    }, 'seed1')).toEqual('Hello to the world');
});

it('Accounts for part weights', () => {
    for(let i = 0; i < 10; i++) {
        expect(textpack([{
            value: 'Hello world',
            weight: 100000,
        }, {
            value: 'Another value',
        }], 'seed' + i)).toEqual('Hello world');
    }
});

it('Throws an error during strict evaluation', () => {
    expect(() => textpack('Hello {abc}', {strict: true})).toThrow('{abc}');
});
